@@ -1,9 +1,14 @@
 ## TODO
 ## * FONTS
+## some common function
 
+## does gtext object have a selection
+hasSelection = function(obj) tclvalue(tktag.ranges(getWidget(obj),"sel")) != ""
+
+## begin
 setClass("gTexttcltk",
-         representation(tags="list"),
-         contains="gComponenttcltk",
+           representation(tags="list"),
+           contains="gComponenttcltk",
          prototype=prototype(new("gComponenttcltk"))
          )
 
@@ -67,10 +72,15 @@ setMethod(".gtext",
             obj <- new("gTexttcltk", block=gp, widget=txt, tags=list(),
               toolkit=toolkit,ID=getNewID(), e = new.env())
 
+            ## font.attr sets text properties for entire buffer
+            if(!is.null(font.attr)) {
+              font(obj) <- font.attr
+            }
+
 
             ## add initial text
             if(!is.null(text)) {
-              add(obj, text, font.attr=font.attr)
+              add(obj, text)
             }
 
             ## set height if requested
@@ -102,9 +112,9 @@ setMethod(".svalue",
             if(!is.null(index) && index == TRUE) {
               ## get the selected text from gtext,
               ## return the index instead of text.
-              if(tclvalue(tktag.ranges(getWidget(obj),"sel")) != ""){
-                val <- strsplit(tclvalue(tktag.ranges(getWidget(obj),
-                  "sel")), " ")[[1]]}
+              if(hasSelection(obj))
+                ## row.column: row 1-based, column 0-based
+                val <- as.character(tktag.ranges(getWidget(obj),"sel"))
               else
                 val <- c(0,0)
               return(as.numeric(val))
@@ -188,6 +198,8 @@ setMethod(".add",
             
             where <- ifelse(is.null(theArgs$where), "end",theArgs$where)
             if(where != "end") where = "0.0" ## the beginning
+
+            txt <- getWidget(obj)
             
             value <- paste(value,collapse="\n")
             if(do.newline)
@@ -195,59 +207,46 @@ setMethod(".add",
 
             ### Handle markup here
             markup <- theArgs$font.attr
-            if(!is.null(markup) && !is.list(markup))
-              markup <- lapply(markup,function(x) x)
             
             if(!is.null(markup)) {
-              ## set up tag for handling markup
-              argList <- list(getWidget(obj),"foo")
-              fontList <- c()
-              if(!is.null(markup$family))
-                fontList <- c(fontList,family=switch(markup$family,
-                                        "normal"="times",
-                                        "sans" = "helvetica",
-                                        "serif" = "times",
-                                        "monospace"="courier",
-                                        markup$family))
-              if(!is.null(markup$weight))
-                fontList <- c(fontList,slant=switch(markup$weight,
-                                        "normal"="normal",
-                                        "oblique"="normal",
-                                        "italic"="italic",
-                                        markup$weight))
-              if(!is.null(markup$style))
-                fontList <- c(fontList, weight=switch(markup$style,
-                                         "bold"="bold",
-                                         "ultra-bold"="bold",
-                                         "heavy"="bold",
-                                         markup$style))
+              ## bit of a hack to set font
+              fname <- paste(as.character(date()),rnorm(1), sep="") ## some random string
+              fontList <- fontlistFromMarkup(markup, fname)
+              do.call("tkfont.create", fontList)
+
+              tkmark.set(txt, "left","insert"); tkmark.gravity(txt,"left","left")
+              tkmark.set(txt, "right","insert"); tkmark.gravity(txt,"right","right")
+              tkinsert(txt, where, value)
+              tktag.add(txt, fname, "left","right")
+              tktag.configure(txt, fname, font=fname)
+              if("color" %in% names(markup))
+                tktag.configure(txt, fname, foreground=markup['color'])
+              ## Color!!
               
-              if(!is.null(markup$size))
-                if(is.numeric(markup$size))
-                  fontList <- c(fontList, size=markup$size)
-                else
-                  fontList <- c(fontList,size = switch(markup$size,
-                                          "xxx-large"=24,
-                                          "xx-large"=20,
-                                          "x-large"=18,
-                                          "large"=16,
-                                          "medium"=12,
-                                          "small"=10,
-                                          "x-small"=8,
-                                          as.integer(markup$size)))
+##               ##              if(!is.null(markup$color))
+##               ##               argList$foreground = markup$color
               
+##               add(obj,"")
+##               tktag.add(txt, "buffer", "0.0", "end")              
 
 
-              if(length(fontList) > 0)
-                argList$font = fontList
 
-              if(!is.null(markup$color))
-                argList$foreground = markup$color
+##               ## old
+##               ## set up tag for handling markup
+##               argList <- list(txt,"foo")
 
-              ## now configure a tag
-              do.call("tktag.configure",argList)
+##               fontList <- fontlistFromMarkup(markup)
 
-              tkinsert(getWidget(obj),where,value,"foo")
+##               if(length(fontList) > 0)
+##                 argList$font = fontList
+
+##               if(!is.null(markup$color))
+##                 argList$foreground = markup$color
+
+##               ## now configure a tag
+##               do.call("tktag.configure",argList)
+
+##               tkinsert(txt, where,value,"foo")
 
               
             } else {
@@ -280,8 +279,24 @@ setMethod(".add",
 setReplaceMethod(".font",
                  signature(toolkit="guiWidgetsToolkittcltk",obj="gTexttcltk"),
                  function(obj, toolkit, ..., value) {
-                   ### XXX Implement this
-                   gwCat("gtext: implement font()\n")
+                   ## if a selection, set font, else set font for buffer
+                   widget <- getWidget(obj)
+                   if(hasSelection(obj)) {
+                     selected <- as.character(tktag.ranges(getWidget(obj),"sel"))
+                     fname <- paste(as.character(date()),rnorm(1), sep="") ## some random string
+                     ## make font, tag in buffer, configure tag
+                     fontList <- fontlistFromMarkup(value)
+                     do.call("tkfont.create", merge(list(fname), fontList))
+                     tktag.add(widget, fname, selected[1], selected[2])
+                     tktag.configure(widget, fname, font=fname)
+                     if("color" %in% names(value))
+                       tktag.configure(widget, fname, foreground=value['color'])
+                   } else {
+                     ## clear out old tags -- we are resetting
+                     tagNames <- as.character(tktag.names(widget))
+                     sapply(tagNames, function(i) tktag.delete(widget, i))
+                     .font(widget, toolkit, ...) <- value
+                   }
                    return(obj)
                  })
 
